﻿using System.Collections.Generic;
using System;
namespace Persistence {
  public class XmlFormatter {

    delegate string Formatter(ISerializationToken token);

    private Dictionary<Type, Formatter> formatters;
    private int depth;

    public XmlFormatter() {
      this.formatters = new Dictionary<Type, Formatter>();
      this.formatters.Add(typeof(BeginToken), new Formatter(FormatBegin));
      this.formatters.Add(typeof(EndToken), new Formatter(FormatEnd));
      this.formatters.Add(typeof(PrimitiveToken), new Formatter(FormatData));
      this.formatters.Add(typeof(ReferenceToken), new Formatter(FormatReference));
      this.formatters.Add(typeof(NullReferenceToken), new Formatter(FormatNullReference));
      this.depth = 0;
    }

    public string Format(ISerializationToken token) {
      return formatters[token.GetType()](token);
    }

    private string Prefix {
      get { return new string(' ', this.depth * 2); }
    }

    private string FormatBegin(ISerializationToken token) {
      BeginToken beginToken = (BeginToken)token;
      string result =
        String.Format("{0}<COMPOSITE name=\"{1}\" type=\"{2}\" id=\"{3}\">\n",
          this.Prefix, beginToken.Accessor.Name, beginToken.Accessor.Get().GetType(), beginToken.Id);
      this.depth += 1;
      return result;
    }

    private string FormatEnd(ISerializationToken token) {
      EndToken endToken = (EndToken)token;
      this.depth -= 1;
      return Prefix + "</COMPOSITE>\n";
    }

    private string FormatData(ISerializationToken token) {
      PrimitiveToken dataToken = (PrimitiveToken)token;
      return String.Format("{0}<PRIMITIVE name=\"{1}\" type=\"{2}\">{3}</PRIMITIVE>\n",
        this.Prefix, dataToken.accessor.Name, dataToken.accessor.Get().GetType(), dataToken.Data);
    }

    private string FormatReference(ISerializationToken token) {
      ReferenceToken refToken = (ReferenceToken)token;
      return String.Format("{0}<REFERENCE name=\"{1}\" ref=\"{2}\"/>\n",
        this.Prefix, refToken.Name, refToken.Id);
    }

    private string FormatNullReference(ISerializationToken token) {
      NullReferenceToken nullRefToken = (NullReferenceToken)token;
      return String.Format("{0}<NULL name=\"{1}\"/>\n",
        this.Prefix, nullRefToken.Name);
    }
  }
}